c  hde4.f
c  1-D heat diffusion equation, general theta-rule solver.
c  Miroslav Broz (miroslav.broz@email.cz), Jul 17th 2007

      program hde4
c
c  constants
c
      include 'const.inc'
      include 'dimen.inc'
c
c  variables
c
      real*8 Temp(0:Nmax),Temp_new(0:Nmax),x(0:Nmax)
      real*8 t,tmin,tmax,dt,dtout,tout,dx,xi,K,C,rho,D,theta
      real*8 A_Bond,epsil,r,T_eq,T_star,P,f,delta,xmax
      real*8 n_perp(3),n_Sun_t0(3),E
      integer N,j
      character*80 s
      complex a(5),z,roots(4),real_roots(4)
      real*8 y

c  functions
      real*8 Insolation,sgn
c
c  read input parameters
c
      write(*,15) '# rho [kg/m^3] = '
15    format(a,$)

5     continue
        read(*,*,err=990,end=990) s
      if (s(1:1).eq.'#') goto 5

      read(s,*,err=990,end=990) rho
      write(*,*) rho

      write(*,15) '# C [J/kg/K] = '
      read(*,*,err=990,end=990) C
      write(*,*) C

      write(*,15) '# K [W/m/K] = '
      read(*,*,err=990,end=990) K
      write(*,*) K

      write(*,15) '# A_Bond [] = '
      read(*,*,err=990,end=990) A_Bond
      write(*,*) A_Bond

      write(*,15) '# epsilon [] = '
      read(*,*,err=990,end=990) epsil
      write(*,*) epsil

      write(*,15) '# P [s] = '
      read(*,*,err=990,end=990) P
      write(*,*) P
      write(*,20) '# P [h] = ', P/3600.d0

      write(*,15) '# r [AU] = '
      read(*,*,err=990,end=990) r
      write(*,*) r

      write(*,15) '# n_perp [x,y,z] = '
      read(*,*,err=990,end=990) n_perp(1),n_perp(2),n_perp(3)
      call normalize(n_perp)
      write(*,*) n_perp(1),n_perp(2),n_perp(3)

      write(*,15) '# n_Sun_t0 [x,y,z] = '
      read(*,*,err=990,end=990) n_Sun_t0(1),n_Sun_t0(2),n_Sun_t0(3)
      call normalize(n_Sun_t0)
      write(*,*) n_Sun_t0(1),n_Sun_t0(2),n_Sun_t0(3)

      write(*,30) '# N [] (Nmax = ',Nmax,') = '
30    format(a,i5,a,$)
      read(*,*,err=990,end=990) N
      write(*,*) N

      write(*,15) '# dx [m] = '
      read(*,*,err=990,end=990) dx
      write(*,*) dx

      write(*,15) '# dt [s] = '
      read(*,*,err=990,end=990) dt
      write(*,*) dt

      write(*,15) '# dtout [s] = '
      read(*,*,err=990,end=990) dtout
      write(*,*) dtout

      write(*,15) '# tmin [s] = '
      read(*,*,err=990,end=990) tmin
      write(*,*) tmin

      write(*,15) '# tmax [s] = '
      read(*,*,err=990,end=990) tmax
      write(*,*) tmax
      write(*,20) '# tmax [h] = ', tmax/3600.d0

      write(*,15) '# theta [] = '
      read(*,*,err=990,end=990) theta
      write(*,*) theta

      goto 995
990   continue
        write(10,991) '# Error: reading input parameters.'
991     format(/,a)
        stop
995   continue

c
c  pre-calculate some parameters
c

c  diffusivity
      D = K/(rho*C)
      write(*,20) '# D [m^2/s] = ', D

c  von Neumann stability criterion
      xi = 2*D*dt/(dx**2)
      write(*,20) '# xi [] = ', xi
20    format(a,f12.5)

      if (xi.gt.1) then
        write(*,10) '# Warning: von Neumann stability criterion is NOT'
     :    // ' fulfilled!'
10      format(a)
      endif

c  equilibrium temperature
c  pi R^2 (1-A) L_Sun / (4 pi r^2) = 4 pi R^2 epsilon sigma T_eq^4
      T_eq = ((1-A_Bond)*S0 / (r**2 * 4*epsil*sigma))**0.25d0
      write(*,20) '# T_eq [K] = ', T_eq

c  subsolar temperature
      T_star = sqrt(2.d0)*T_eq
      write(*,20) '# T_star [K] = ', T_star

c  diurnal/seasonal frequency
      f = 1.d0/P
      write(*,20) '# f [Hz] = ', f
      write(*,20) '# f [rev/day] = ', day/P

c  penetration depth
      delta = sqrt(D/(pi*f))
      write(*,20) '# delta [m] = ', sgn(dx)*delta

      xmax = N*dx
      write(*,20) '# xmax [m] = ', xmax
      write(*,20) '# xmax [delta] = ', xmax/delta
      if (abs(xmax).lt.(3*delta)) then
        write(*,10) '# Warning: maximum depth xmax is smaller than'
     :    // ' 3*delta (i.e., the penetration depth of the thermal'
     :    // ' wave)!'
      endif

c  spatial coordinates
      do j = 0, N
        x(j) = j*dx
      enddo
c
c  initial conditions (t=0)
c
      do j = 0, N
        Temp(j) = T_eq
      enddo
c  subsolar temperature at the surface
c      Temp(0) = T_star
c
c  an initial dump
c
      write(*,10) '# t [s] & x [m] & T [K]'
      t = tmin
      tout = tmin + dtout
      E = insolation(t,f,r,n_perp,n_Sun_t0)
      write(*,20) '# t [s] = ', t
      write(*,20) '# E [W] = ', E
      do j = 0, N
        write(*,*) t, x(j), Temp(j)
      enddo
      write(*,*)
c
c  main loop
c
      do while (t.le.(tmax - 0.5d0*dt))

        t = t + dt

c  boundary condition (j=0)
c  K (dT/dr)_surface + epsilon sigma T^4 = (1-A) * E(t) * n_perp(r)
        E = insolation(t,f,r,n_perp,n_Sun_t0)

c  for K<>0 we have to solve a 4-th order polynomial (by Laguerre method)
        a(1) = cmplx( - (1.d0-A_Bond)/(epsil*sigma) * E
     :    + K/(epsil*sigma) * 1./dx * Temp(1) )
        a(2) = cmplx( - K/(epsil*sigma) * 1./dx )
        a(3) = (0.,0.)
        a(4) = (0.,0.)
        a(5) = (1.,0.)
c  subsolar temperature as a 1st guess
        T_star = ((1.d0-A_Bond) * E / (epsil*sigma))**0.25d0
        z = cmplx(T_star)
        call laguer(a,4,z,j)
        Temp(0) = dble(z)

c  boundary condition (j=N)
c  T = T_eq, NOT dT/dt = 0
        Temp(N) = T_eq

        call theta_rule_step(Temp,N,dt,dx,D,theta,Temp_new)

        do j = 0, N
          Temp(j) = Temp_new(j)
        enddo

c  print output every dtout
        if (t.ge.(tout-0.5d0*dt)) then
          write(*,20) '# t [s] = ', t
          write(*,20) '# E [W] = ', E
          do j = 0, N
            write(*,*) t, x(j), Temp(j)
          enddo
          write(*,*)
          tout = tout + dtout
        endif

      enddo

      stop
      end


